
#************************************************************************
#
# Author: Tarun Goyal  
#
# Module: menubar_manager.tcl
#
# Descr.: MenuManager is a central repository which will entertain
#         request that is recieved from any window of the framework, for 
#         displaying the relevant menus of a menu-button
#         
# All Rights Reserved
#
#***********************************************************************/




# MenuManager is a central repository which will entertain
# request that is recieved from any window of the framework.

itcl::class MenuBarManager {
   #variable list
          
   # Following data-structures are maintained by Task-Manager internally ::
   
   # these store the dock-button related properties
   private variable globalMenuButtons
   private variable menuButtonMenus
   private variable menuProps
   private variable menuButtonProps

   # List of created menu-buttons are maintained as ::
   # createdMenuButtons {} = { std, .................... }
   private variable  createdMenuButtons {}
          
   # Array which tells us about the status of a menu-button ::
   # menuButtonStatus (std) = { "Enable" } ...............
   private variable  menuButtonStatus
          
   # Global list of registered menu-buttons and menus ::
   # Reg_WindowMenuButton (docName) = {  windowName, ...........}
   # Reg_WindowMenu (docName, ButtonName) = { windowName, ........................}  
   # Reg_WindMenuCallback(docName, ButtonName) = {  CallbackFunction , ........... }
   private variable  Reg_WindowMenuButton 
   private variable  Reg_WindowMenu
   private variable  Reg_WindMenuCallback
   
   # this will store the status of a menu at a given stage
   private variable menuItemStatus

   # this list will store the menus that you want to create dynamically
   # at each update. e.g. display->marking menu (format = [list "menubutton,menu" ..])
   private variable menuIsDynamic

   # store the current window name, used by MenuItemValidForMarking
   private variable currentWindow

   # Menu-BAr associated with a window:
   # windowMenuBar(windowName) = {menuBarObject}
   private variable  windowMenuBar
   
   # this will store the global tool-bar object of the main framework
   private  variable  globalMenuBarObject       
   
   # this will store dftFramework's object
   private variable frameworkHandle

   # this will store keyIndex array
   private variable keyIndexArray 
   
   #methods
   constructor { menuBarName frameworkObj } {
      global _vars
      global _fonts
      
      set frameworkHandle $frameworkObj

      ## call to initialize various lists
      initializeLists
      
      set globalMenuBarObject [ iwidgets::menubar $_vars(debug_win).$menuBarName -font $_fonts(helvB:12) -helpvariable helpstr ]
      foreach menuButton $globalMenuButtons {
         set menuButtonStatus($menuButton) ""
         foreach menuName $menuButtonMenus($menuButton) {
            set menuItemStatus($menuButton,$menuName) "delete"
         }
      }
      
      set i 0
      while { $i < [llength $globalMenuButtons] } {
         set createdMenuButtons [linsert $createdMenuButtons $i " "]
         incr i
      }
      
   }
   
   # this function initializes the various dock-bar related Lists
   private method initializeLists {}
   
   # Register_Window::  This  function enables a window to register associated 
   # doc-Name and button-Name
   public method  Register_Window  { windowName menuButtonName menuName callBackFunc }
          
   # Unregister_Window:: This function deregisters the given window from the 
   # both the lists :: Reg_WindowMenuButton and Reg_WindowMenu.
   public method  Unregister_Window {windowName }

   # Update_MenuBar :: This proc is  called by dftFramework and  depending on
   # the windowName updates the respective menu-bar. Showing/hiding the menu-bar 
   # is also handled here.
   public method Update_MenuBar  { windowName Docked {forceTag "no"}}
     
   # Update_MenuButton :: This function updates the requested menu-button, with the 
   # latest-status of the menus residing inside the menu-button.
   private method  Update_MenuButton { menuButtonName object windowName menuBarObjState {forceTag "no"}}

   # Create_MenuButton :: Create a menu-button with a given menuButtonName..
   private method  Create_MenuButton { menuButtonName menuBarObj Docked {enterTag 0} } 
 
   # getUndockedMenuBarObj  :  This proc creates the object for undocked menu-bar with all the 
   # contents that goes into the Undocked Window. This proc is called as soon as the window is undocked.
   public method GetUndockedMenuBarObj { windowName  }
          
   # this will handle the call-backs
   public method CallBackHandler { menuName menuButtonName }
          
   #  this  method  changes the state of menus  depending  upon
   # the current selection inside a window
   public method UpdateState { state windowName menuButtonName { menuName "" } }
   
   # this method handles tool-bars when a window is maximized
   public method maximizeWindowMenuBars { windowName }
   
   # method creates deleted menu-item
   private method Create_DeletedMenuItem { menuButtonName menuName menuObj insertIndex Docked windowName {enterTag 0}}

   # return global tool bar object
   public method getGlobalMenuBarObj {} {
      return $globalMenuBarObject
   }
   # function to test for the validity of menu-item in "data" menu-button for "design & debug"
   public method MenuItemValidForData { args }

   # function to test for the validity of menu-item in "data" menu-button for "browser window"
   public method MenuItemValidForBrowser { args }

   # function to test for the validity of menu-item in "Display->Marking" for "schematic windows"
   public method MenuItemValidForMarking { args }

   # this function sets the widget value to "value"
   public method setWidgetVal { windowName menuButtonName menuName value }

   # this function returns the value stored in widget
   public method getWidgetVal { windowName menuButtonName menuName }
   
   # this function sets the key index array
   public method setKeyIndex {menuNameList }

   # this function returns the key Index
   public method getKeyIndex { menuName }
   
   
}

## FOLLOWING METHOD CONTAINS SOME EXAMPLES OF THE WAY YOU SHOULD DEFINE THE WIDGET PROPERTIES.
itcl::body MenuBarManager::initializeLists {} {
   global _vars
   
   ## menuButtonMenus -- list of all the menus inside a menu-button
   ## syntax  for "menuButtonProps" --- { Text for MenuButton , Underlying Alphabet index }
   ##menu Props :: syantax :: { label , Underlying Alphabet index, State[cascade,normal] Function to be called for menu's validity[optional] }

   ## for CHECKBUTTONS menuProps syntax :: { label , Underlying Alphabet index, checkbutton , variablename, Function to be called for menu's validity[optional] }
   ## for RADIOBUTTONS menuProps syntax :: { label , Underlying Alphabet index, radiobutton , variablename, valueofvariable Function to be called for menu's validity[optional] }
   
   set globalMenuButtons              { file edit display trace data tools framework help}
   
   ## common menu buttons as-per spec
      set menuButtonMenus(file) { open saveAs createDofile print sep1 close sep2 close_window}

      set menuButtonMenus(display) { add delete sep1 gatelevel sep2 find sep3 collapseAll sep4 expand collapse sep5 crossSelect sep6 zoom marking compaction sep7 view hierExp netBundle groupInst displayNets sep8 testSetup cursor sep9 grouping filter}
   

   set menuButtonMenus(trace) { bk bkEnd fwOne fwFanout fwEnd upOne downOne Down}
   
   set menuButtonMenus(data) { noData testSetup loadUnload shift skewLoad shadowControl masterObserve shadowObserve stateStability errorPattern assert k19 k22 vcd patternIndex parallelPattern record fault tied constrain trace seqDepth clockCone delayData gates primitives totFaults undetFaults faults drc coverage clkAtt refresh namedcapture falsepath testEnd}
   

   set menuButtonMenus(tools) { anDrc anFaults anMismatch sep1 diagReport invokeLayout }
   set menuButtonMenus(help) { startUp visCommandRef refManual infoHub sep1 mouseStrokes keyBinds sep2 about}
      set menuButtonMenus(framework)     { taskMgr debug hierSch hierBrowser sigBrowser dataview waveview findWindow formatGuide tsv transcript sep close} 
      set menuButtonMenus(edit)          { undo redo sep1 cut copy paste selectAll sep2 pref}

   set menuButtonProps(framework) { "Windows" 0 }
   set menuButtonProps(dataview) { "Dataview" 0 }
   set menuButtonProps(browser_options)  { "Browser Options" 0 }
   set menuButtonProps(signal_options)  { "Signal Options" 2 }
   set menuButtonProps(edit) { "Edit" 0 }

   set menuButtonProps(file)  { "File" 0 }
   set menuButtonProps(display)  { "Display" 1 }
   set menuButtonProps(trace)  { "Trace" 1 }
   set menuButtonProps(data)  { "Data" 1 }
   set menuButtonProps(tools)  { "Tools" 1 }
   set menuButtonProps(help)  { "Help" 0 }
   set menuButtonProps(view)  { "Display" 0 }     
        
   ## hier-browser window specific sub-menus
   set menuButtonMenus(data,coverage)  { test_coverage rel_test_coverage test_coverage_loss fault_coverage atpg_effectiveness}
   #set menuButtonMenus(data,libCoverage)  {test_coverage test_coverage_loss fault_coverage}
   
   ## dataview window cascaded buttons
   set menuButtonMenus(dataview,delete) { deleteRow }
   set menuButtonMenus(dataview,dataSet) { patternIndex drcPat }
   set menuButtonMenus(dataview,dataSet,drcPat) { testSetup loadUnload shift skewLoad shadowControl masterObserve shadowObserve sep1 stateStability testEnd }
      
   ## display menu-button's cascaded sub-menus
   set menuButtonMenus(display,zoom)   { zoomAll zoomIn zoomOut zoomSelected zoomMarked }
   set menuButtonMenus(display,filter) { error warning command all }
   #set menuButtonMenus(display,delete) { deleteAll deleteSelected }
   set menuButtonMenus(display,view) { viewAll viewSelected viewMarked }  
   # set menuButtonMenus(display,mark) { markAll markSelected }
   # set menuButtonMenus(display,unmark) { unmarkAll unmarkSelected }
   set menuButtonMenus(display,marking) { color1 color2 color3 color4 color5 unmark }
   # set dynamic property to true in order to force recreate the menu in each update
   set menuIsDynamic(display,marking) 1
   set menuButtonMenus(display,gatelevel) { design primitive }
   set menuButtonMenus(display,compaction) {on off}
   
      ##Handle Testability_data Menu
   
   ##Handle DRC Menu
   set menuProps(data,drc)                       { "DRC" 0 "cascade" "MenuItemValidForBrowser drc All"}
   set drcList [c_HbGetDRCRuleNames]
   list menuButtonMenus(data,drc) 
   lappend menuButtonMenus(data,drc) "all"
   lappend menuButtonMenus(data,drc) "delall"
   foreach Itr $drcList {
      lappend menuButtonMenus(data,drc) $Itr
   }
   set menuProps(data,drc,all)                   { "Add All" 0 "normal" "MenuItemValidForBrowser drc All" }
   set menuProps(data,drc,delall)                { "Delete All" 0 "normal" "MenuItemValidForBrowser drc All" }
   foreach  Itr $drcList {
      list menuProps(data,drc,$Itr) 
      lappend menuProps(data,drc,$Itr) $Itr "0" "checkbutton" "drc,$Itr" "MenuItemValidForBrowser drc $Itr"
   }

    ## delete sub-menus
 }

 
itcl::body MenuBarManager::Register_Window { windowName menuButtonName menuName callBackFunc } {
   set enterFlag 0
   
   # Firstly check for the validity of the three arguments
      
   # If the "windowName" does not exist in the "docName"
   # we add it to the "docName" as follows
   if  { ![info exists Reg_WindowMenuButton($menuButtonName)] || \
         ![info exists Reg_WindowMenuButton($menuButtonName,$menuName)] } {
      
      set enterFlag 1
      
      if { ![info exists Reg_WindowMenuButton($menuButtonName)] } {
         lappend Reg_WindowMenuButton($menuButtonName) $windowName 
      } elseif { [info exists Reg_WindowMenuButton($menuButtonName)] && \
                 [lsearch $Reg_WindowMenuButton($menuButtonName) $windowName] == -1 } {
         lappend Reg_WindowMenuButton($menuButtonName) $windowName
      }

      #next we add the button name to the respective window name
      lappend Reg_WindowMenu($menuButtonName,$menuName) $windowName
            
      # add the call-back function to the array
      lappend Reg_WindMenuCallback($menuButtonName,$menuName) $callBackFunc
   }  
       
   # Now we see whether there already exists a  menu-button
   # with the given "docbarName", if not, we create it
   if { $enterFlag == 1 } {
      if { [lsearch $createdMenuButtons $menuButtonName]  == -1 } {
         # if docBar not found
         set menubutton_command [Create_MenuButton $menuButtonName $globalMenuBarObject "docked"]
         set menuButtonObj [eval $menubutton_command]
      }
   } 
}
  
itcl::body MenuBarManager::Unregister_Window { windowName } {
   ## special handling of separators
   if { [info exists windowMenuBar($windowName)] } { 
      ## window is docked
      set object $windowMenuBar($windowName)
      itcl::delete object $object
      set $windowMenuBar($windowName) ""
   } else {
      ## window is docked
      set object $globalMenuBarObject
      #  Unregister the "WindowName" by taking it out
      #  from the list Reg_WindowMenu & Reg_WindowMenuButton
      foreach menuButton $globalMenuButtons {
         if { [info exists Reg_WindowMenuButton($menuButton)] } {
            set index  [lsearch $Reg_WindowMenuButton($menuButton) $windowName]
            if { $index != -1 } {
               # search the windowName over the entire button-list
               foreach menuName $menuButtonMenus($menuButton) {
                  if { [info exists Reg_WindowMenu($menuButton,$menuName) ] } {
                     set button_index [lsearch $Reg_WindowMenu($menuButton,$menuName) $windowName]          
                     if {  $button_index != -1} {
                        set  Reg_WindowMenu($menuButton,$menuName) \
                        [lreplace $Reg_WindowMenu($menuButton,$menuName) $button_index $button_index]
                     }
                  }
               }
               set  Reg_WindowMenuButton($menuButton) [lreplace $Reg_WindowMenuButton($menuButton) $index $index]
            }
        
            #  Delete the "menu-bar" object, if we have emptied the list
            if { [llength $Reg_WindowMenuButton($menuButton)] == 0 } {
              $object delete $menuButton $menuButton  
            }
         }
      } ; #end of foreach   
   }
   Update_MenuBar $windowName "docked" 
}
 
itcl::body MenuBarManager::Update_MenuBar { windowName Docked {forceTag "no"}} {

   set paneManager [$frameworkHandle getPaneManager]
    
   #  Here we update the menu-bar asoociated with the
   #  window "windowName" , that has undergone a
   #  change both for docked and un-docked window
     
   if { $Docked == "docked" } {
      set obj $globalMenuBarObject
   } else {
      set obj $windowMenuBar($windowName)
   }
   
   #  Get the list of all menu-buttons associated with this
   #  window and see the status of associated doc-bars
   # and display them accordingly
   foreach menuButtonName $globalMenuButtons {
      set isMenuButtonNeeded FALSE
      
      # check for if window is docked and menu-button" is needed
      if { [info exists Reg_WindowMenuButton($menuButtonName)] } {
         if { ( $Docked == "docked" ) && ( [lsearch $Reg_WindowMenuButton($menuButtonName) $windowName] != -1  ||  \
               [lsearch $Reg_WindowMenuButton($menuButtonName) "frameworkWindow" ] != -1 ) } {
            set  isMenuButtonNeeded TRUE
         } elseif { [lsearch $Reg_WindowMenuButton($menuButtonName) $windowName] != -1 } {
            # window is undocked but dockbar needed    
            set  isMenuButtonNeeded TRUE
         } 
      }    
      
      
      ## here we find the index for the given 
      ## menu-button name
      set index 0
      set end [ $obj index end] 
      set required "no"
      set tempVal -1
      while {$index <= $end} {
         set tempVal [ $obj index $menuButtonName]
         incr index
      }
      

      ## here if the particular menu-button was "hidden" 
      ## then also required is true
      if { $tempVal != -1 || ( $isMenuButtonNeeded == "TRUE" && \
           $menuButtonStatus($menuButtonName) == "hide" ) } {
         set required "yes"
      }
      
      # here we check whether there is a need
      # to hide the menu-button
      set hideTag 1
      if { [info exists Reg_WindowMenuButton($menuButtonName) ] } {
         foreach window $Reg_WindowMenuButton($menuButtonName) {
            if { ( $window != "frameworkWindow" ) && \
                 ([$paneManager state "$paneManager.$window" ] != "hidden" && \
                  [$paneManager state "$paneManager.$window" ] != "undocked")  } {
               set hideTag 0
            } elseif { $window == "frameworkWindow" } {
               set hideTag 0
            } elseif { $window != "frameworkWindow" && $Docked == "undocked" && \
                       [$paneManager state "$paneManager.$window" ] == "undocked" } {
               set hideTag 0
            }
         }
      }
      
      # special case of "hiding" the menu-button
      if { $hideTag == 1 && $menuButtonStatus($menuButtonName) != "hide" } {      
         ## delete the menubutton
         if { $required == "yes" } {
            ## we need to delete all the associated menus
            ## before deleting the menu-button  
            set menuObj "$obj.menubar.$menuButtonName.menu"
            set end [$menuObj index last]
            if { $end != "none" } {
               for {set index 0} {$index<=$end} {incr index} {
                  $menuObj delete $index $index
               }
            }
            ## very important :: need to set the associated menus 
            ## status to delete
            foreach menuName $menuButtonMenus($menuButtonName) {
               set menuItemStatus($menuButtonName,$menuName) "delete"
            }  
            $obj delete $menuButtonName
            
            #removing menu-button from created menu-buttons list
            set index [lsearch $createdMenuButtons $menuButtonName]
            if { $index != -1 } { 
               set createdMenuButtons [lreplace $createdMenuButtons $index $index]
               set createdMenuButtons [linsert $createdMenuButtons $index " "]
            }

            # setting the status
            set menuButtonStatus($menuButtonName) "hide"
         }
      }
      
      if { $isMenuButtonNeeded == "TRUE" && $hideTag == 0 } {
                  
         if { $required == "yes"  } {
            ## show the menu-button
             
            ## if the menubutton was hidden, then we will 
            ## have to create a new-one, since we have deleted it
            if { $menuButtonStatus($menuButtonName) == "hide" && \
                 [lsearch $createdMenuButtons $menuButtonName]  == -1 } {
               set menubutton_command [Create_MenuButton $menuButtonName $globalMenuBarObject $Docked]
               set menuButtonObj [eval $menubutton_command]
            }

            $obj menuconfigure $menuButtonName -state normal
            set menuButtonStatus($menuButtonName) "enabled"
            
            if { $Docked == "docked" } {
               set menuBarObjState "global"
            } else { 
               set menuBarObjState "local"
            }
            Update_MenuButton $menuButtonName $obj $windowName $menuBarObjState $forceTag
         } elseif { $required == "no" && $menuButtonStatus($menuButtonName) != "disabled" } {
            ## disable the menu-button
            
            ## if the menubutton was hidden, then we will 
            ## have to create a new-one, since we have deleted it
            if { $menuButtonStatus($menuButtonName) == "hide" && \
                 [lsearch $createdMenuButtons $menuButtonName]  == -1 } {
               set menubutton_command [Create_MenuButton $menuButtonName $globalMenuBarObject $Docked]
               set menuButtonObj [eval $menubutton_command]
            }
            
            $obj menuconfigure $menuButtonName -state "disabled"
            set menuButtonStatus($menuButtonName) "disabled"
         }    
      } elseif { $hideTag == 0  && $Docked == "docked" } {
         if { $menuButtonStatus($menuButtonName) != "disabled" } {
            # disable the menu-button
            
            ## if the menubutton was hidden, then we will 
            ## have to create a new-one, since we have deleted it
            if { $menuButtonStatus($menuButtonName) == "hide" && \
                 [lsearch $createdMenuButtons $menuButtonName]  == -1 } {
               set menubutton_command [Create_MenuButton $menuButtonName $globalMenuBarObject $Docked]
               set menuButtonObj [eval $menubutton_command]
            }
            
            $obj menuconfigure $menuButtonName -state "disabled"

            set menuButtonStatus($menuButtonName) "disabled"
         } 
      }
   } ; # end of foreach

}  

itcl::body MenuBarManager::Update_MenuButton { menuButtonName object windowName menuBarObjState {forceTag "no"}} {
   
   # store the current active window to be used
   # in MenuItemValidForMarking
   set currentWindow $windowName
   
   set paneManager [$frameworkHandle getPaneManager]
   set menuObj "$object.menubar.$menuButtonName.menu"
   set firstTimeMenusCreation "false"
   set menuItemIndex -1
   set deleteTag "false"
   
   ## initial checking
   set end [$menuObj index last]
   if { $end == "none" || $menuBarObjState == "local" } {
      set firstTimeMenusCreation "true"
      if { $menuBarObjState == "local" } {
         set index 0
         set end [$menuObj index last]
         if { $end != "none" } {
            $menuObj delete 0 last
         }
      }
   }
  
   ## we are updating menus associated with the a given 
   ## menu-button here. If this menu-button is being populated 
   ## for the first time, we simply create the registered 
   ## menus without worrying about anything else

   if { $firstTimeMenusCreation == "false" } { 
      ## deleting all the separators first
      set index 0
      set end [$menuObj index last]
      while { $index <= $end } {
         if { [$menuObj type $index] == "separator" } {
            $menuObj delete $index $index
         }
         incr index
      }

      # show the menu-button with the latest menu-status
      # menuName is the actual menu item, such as open, save as, close, etc.
      foreach menuName $menuButtonMenus($menuButtonName) {
         set end [$menuObj index last]
         if { $end == "none" } {
            Update_MenuButton $menuButtonName $object $windowName $menuBarObjState
            return
         }
         set deleteTag "false"
         
         if { ( [ info exists Reg_WindowMenu($menuButtonName,$menuName) ] ) } { 
            ## if window is docked there is a need of an 
            ## extra check of "frameworkWindow"
            if { [lsearch $Reg_WindowMenu($menuButtonName,$menuName) $windowName] != -1 || \
                 ( ( $menuBarObjState == "global" ) && \
                 [lsearch $Reg_WindowMenu($menuButtonName,$menuName) "frameworkWindow"] != -1 ) } {
               # that menu was registed, hence Enable the menu
               set index -1
            
               ## we are checking whether we need the menu here
               while {$index <= $end} {
                  if { $index == -1 } {
                     incr index
                  }
                   
                  set tempMenuName [$menuObj entrycget $index -label]
                  if { [lindex $menuProps($menuButtonName,$menuName) 0] == $tempMenuName || \
                       ( [info exists menuItemStatus($menuButtonName,$menuName)] && \
                       $menuItemStatus($menuButtonName,$menuName) == "delete" ) } {
                     incr menuItemIndex

                     # Check if the menu is dynamic, if yes, then reconstruct the menu even if it is present already
                     set isDynamic 0
                     if {[info exists menuIsDynamic($menuButtonName,$menuName)] && $menuIsDynamic($menuButtonName,$menuName) == 1} {
                        set isDynamic 1
                     }
                     if { $forceTag == "yes" || $isDynamic || \
                          ( [info exists menuItemStatus($menuButtonName,$menuName)] && \
                            $menuItemStatus($menuButtonName,$menuName) == "delete" ) } {

                        if { $menuItemStatus($menuButtonName,$menuName)== "added" } {
                             $menuObj delete $index $index
                             set menuItemStatus($menuButtonName,$menuName) "delete" 
                        }
                        
                        set menu_command [Create_DeletedMenuItem $menuButtonName $menuName $menuObj $menuItemIndex $menuBarObjState $windowName]
                        set temp [eval $menu_command]
                        
                        ## menuItemStatus get set only if menu is indeed created
                        if { $menu_command != "" } {
                           set menuItemStatus($menuButtonName,$menuName) "added"
                        }
                     }
                     break
                  }
                  
                  incr index 
               }
            }  else {
               set deleteTag "true"
            }
         }  else {
            set deleteTag "true"
         }
         
         ## deleting the menu-item if tag is true
         if { $deleteTag == "true" } {
            set index 0
            while {$index <= $end} {
               set tempMenuName [$menuObj entrycget $index -label]
               if { [lindex $menuProps($menuButtonName,$menuName) 0] == $tempMenuName } {
                    $menuObj delete $index $index
                  set menuItemStatus($menuButtonName,$menuName) "delete"
                  break
               }
               incr index
            }
         }
      } ; # end foreach
   } else {
      ## creating menus for the first time
      ## of the given menu-button
      foreach menuName $menuButtonMenus($menuButtonName) {
         if { ( [ info exists Reg_WindowMenu($menuButtonName,$menuName) ] ) } { 
            ## if window is docked there is a need of an 
            ## extra check of "frameworkWindow"
            if { [lsearch $Reg_WindowMenu($menuButtonName,$menuName) $windowName] != -1 || \
                 ( ( $menuBarObjState == "global" ) && \
                 [lsearch $Reg_WindowMenu($menuButtonName,$menuName) "frameworkWindow"] != -1 ) } {
               
                incr menuItemIndex
                        
                set menu_command [Create_DeletedMenuItem $menuButtonName $menuName $menuObj $menuItemIndex $menuBarObjState $windowName]
                set temp [eval $menu_command]
                
                ## menuItemStatus get set only if menu is indeed created
                if { $menu_command != "" } {
                   set menuItemStatus($menuButtonName,$menuName) "added"
                }
            } ; #end first-if 
         }  ; #end second-of
      } ; # end foreach
   }
  

   ## next we put the separators at the right place
   set end [$menuObj index last]
   set menuIndex 0
   set toBeInsertSepList {}

   if { $end != "none" } {
      while { ($menuIndex < $end) } {
     
         set currMenuItem [$menuObj entrycget $menuIndex -label]
         set nextMenuItem [$menuObj entrycget [expr $menuIndex + 1] -label ] 
     
         set menuNameIndex 0

         foreach menuName $menuButtonMenus($menuButtonName) {
            incr menuNameIndex 
            if { [lindex $menuProps($menuButtonName,$menuName) 0] == $currMenuItem } {      
               break
            }
         }
      
         while { $menuNameIndex < [llength $menuButtonMenus($menuButtonName)] } {
            set menuName [lindex $menuButtonMenus($menuButtonName) $menuNameIndex ]
            if { [lindex $menuProps($menuButtonName,$menuName) 0] == $nextMenuItem } {
               break
            } elseif { [lindex $menuProps($menuButtonName,$menuName) 2] == "separator" } {
               lappend toBeInsertSepList $nextMenuItem
               break
            }
            incr menuNameIndex
         }  
         incr menuIndex
      }   
   }
    
   foreach menuLabel $toBeInsertSepList {
       set index 0 
       set end [$menuObj index last]
       while { $index <= $end } {
          if { [$menuObj type $index] != "separator" } {
             if { [$menuObj entrycget $index -label] == $menuLabel } {
                $menuObj insert $index separator
                break
             }
          } 
          incr index 
       }  
   }
   
}

itcl::body MenuBarManager::Create_MenuButton { menuButtonName menuBarObj Docked {enterTag 0}}  {
   ## this is for checking whether we are doing for the first time
   set first 0
   set menubutton_command ""
   set nextMenuButton ""
   set indexInGlobalList [lsearch $globalMenuButtons $menuButtonName]

   ## it is only for the docked case we have to worry
   ## about the positions of menu-buttons
   ## this is kinda tricky so please change this very carefully
   if { $Docked == "docked" } {
      if { $indexInGlobalList != -1 && \
           [lsearch $createdMenuButtons $menuButtonName] == -1 } {
         set createdMenuButtons [lreplace $createdMenuButtons $indexInGlobalList $indexInGlobalList]
         set createdMenuButtons [linsert $createdMenuButtons $indexInGlobalList $menuButtonName]
      }
      
      ## search for the next valid element in the array
      set index [expr $indexInGlobalList + 1]
      while { $index < [llength $createdMenuButtons] } {
         if { [lindex $createdMenuButtons $index] != " " } {
            set nextMenuButton [lindex $createdMenuButtons $index]
            break
         }
         incr index
      }
   
      if { $enterTag == 0 && $nextMenuButton == "" } {
         set menubutton_command "$menuBarObj add menubutton $menuButtonName \
                             -text \"[lindex $menuButtonProps($menuButtonName) 0]\" \
                             -underline [lindex $menuButtonProps($menuButtonName) 1]"
      } elseif { $enterTag == 0 && $nextMenuButton != "" }  {
         set menubutton_command "$menuBarObj insert $nextMenuButton menubutton $menuButtonName \
                             -text \"[lindex $menuButtonProps($menuButtonName) 0]\" \
                             -underline [lindex $menuButtonProps($menuButtonName) 1]"
      }
   } elseif { $Docked == "undocked" } {
      if { $enterTag == 0 } {
         set menubutton_command "$menuBarObj add menubutton $menuButtonName \
                             -text \"[lindex $menuButtonProps($menuButtonName) 0]\" \
                             -underline [lindex $menuButtonProps($menuButtonName) 1]"
      }
   }
   
   return $menubutton_command;
}


proc destroyMenu {menuName} {
   if { [winfo exists $menuName] } {destroy $menuName}
}

itcl::body MenuBarManager::Create_DeletedMenuItem { menuButtonName menuName menuObj insertIndex Docked windowName {entryTag 0} }  {

   global _fonts
   set underscore "_"

   set menuLabel      [lindex $menuProps($menuButtonName,$menuName) 0]
   set menuUnderscore [lindex $menuProps($menuButtonName,$menuName) 1]
   set menuCategory   [lindex $menuProps($menuButtonName,$menuName) 2]
   set menuTail       [lindex $menuProps($menuButtonName,$menuName) 3]
   set menuValidation [lindex $menuProps($menuButtonName,$menuName) 4]
   
   ## this is for "radiobutton"
   if { $menuCategory == "radiobutton" || $menuCategory == "checkbutton"  } {
      global var$underscore$menuTail
      global var$underscore$menuTail$underscore$windowName
   } 

   set commandString [list $this "CallBackHandler" $menuName $menuButtonName ]
   set menu_command ""
   set interMenuCommand ""
   set menuItemNeeded "yes"
   set windowState "normal"
   set paneManager [$frameworkHandle getPaneManager]
   
   ## setting the window state here
   if { $windowName != "frameworkWindow" } {
      set windowManaged [$paneManager panes]
      if { $windowManaged != "" } {
         set windowState [ $paneManager state "$paneManager.$windowName" ]
      } else {
         set windowState "normal" 
      }
   } else {
      set windowState "normal"
   }

   ## checking whether we need menu-item or not 
   if { ($menuTail != "") && ( $menuCategory != "radiobutton" && $menuCategory != "checkbutton" ) } {
      set menuItemNeeded [eval $menuTail]
   } elseif {$menuCategory == "checkbutton" && $menuValidation != ""} {
      set menuItemNeeded [eval $menuValidation]
   }
   
   ## creating menu-item is needed  
   if { $menuItemNeeded == "yes" } {

      ## handling five different cases here viz::
      ## normal cascade checkbutton radiobutton separator
      if {  $menuCategory == "normal" } {
         if { $entryTag == 0 } {      
            set menu_command [list $menuObj insert $insertIndex command \
                        -label $menuLabel -underline $menuUnderscore -command $commandString]
         } else {
            set menu_command [list add command \
                            -label $menuLabel -underline $menuUnderscore -command $commandString]
         }
      } elseif { $menuCategory == "cascade" } {
         ## recursion, this is for cascading menuName
         if { $entryTag == 0 } {
            set menu_command  [list $menuObj insert $insertIndex cascade -label $menuLabel -underline $menuUnderscore -menu ${menuObj}.${menuName}${Docked}]
         } else {
            set menu_command  [list add cascade -label $menuLabel -underline $menuUnderscore -menu ${menuObj}.${menuName}${Docked}]
         } 
         
         set newMenuButton ${menuButtonName},${menuName}
         set createdMenu ${menuObj}.${menuName}${Docked}
         set menuObj ${menuObj}.${menuName}${Docked}
         # changing the menuObj here
         
         destroyMenu $createdMenu
         set menuHandle [menu $createdMenu -font $_fonts(helvB:12)]
 
         ## creating the sub-menus here
         foreach menus $menuButtonMenus($newMenuButton) { 
            set tempCommand [Create_DeletedMenuItem $newMenuButton $menus $menuObj $insertIndex $Docked $windowName 1]
            if { $tempCommand != "" } {
               set interMenuCommand "$menuHandle $tempCommand"
               set temp [eval $interMenuCommand]
            } 
         }
      
      } elseif { $menuCategory == "checkbutton" } {
         if { $windowState != "undocked" } {
            set variableName "var$underscore$menuTail"
         } else {
            set variableName "var$underscore$menuTail$underscore$windowName"
         } 
         if { $entryTag == 0 } {
            set menu_command [list $menuObj insert $insertIndex checkbutton \
                           -label $menuLabel -underline $menuUnderscore -onvalue on -offvalue off -variable $variableName -command $commandString]
         } else {
            set menu_command [list add checkbutton \
                           -label $menuLabel -underline $menuUnderscore -onvalue on -offvalue off -variable $variableName -command $commandString]
         }
      } elseif { $menuCategory == "radiobutton" } {
         if { $windowState != "undocked" } {
            set variableName "var$underscore$menuTail"
         } else {
            set variableName "var$underscore$menuTail$underscore$windowName"
         }
         
         if { $entryTag == 0 } {
            set menu_command [list $menuObj insert $insertIndex radiobutton \
                        -label $menuLabel -underline $menuUnderscore -value $menuValidation -variable $variablename -command $commandString]
         } else {
            set menu_command [list add radiobutton \
                        -label $menuLabel -underline $menuUnderscore -value $menuValidation -variable $variableName -command $commandString]
         }
      } elseif { $menuCategory == "separator" } {
         set menuTail [lindex $menuProps($menuButtonName,$menuName) 3]
         if { $windowState != "undocked" } {
            set variableName "var$underscore$menuTail"
         } else {
            set variableName "var$underscore$menuTail$underscore$windowName"
         }
         if { $entryTag == 0 } {
            set menu_command [list $menuObj insert $insertIndex separator]
         } else {
            set menu_command [list add separator]
        }
     } 
   }
   return $menu_command
}
  

# this proc will create a new menu-bar-object for the
# given window name and returns it 
itcl::body MenuBarManager::GetUndockedMenuBarObj { windowName } {
   global _vars
   global _fonts
   
   # if the menu-bar object exits for the given window then,
   # return it, else create a new one
   if { ![info exists windowMenuBar($windowName)] } {
      set object [ iwidgets::menubar "[$frameworkHandle getPaneManager].$windowName.menubar" -font $_fonts(helvB:12) -helpvariable helpstr ]
         
      # next we see what all "new" menu-buttons will go into the window
      foreach menuButtonName $globalMenuButtons {
         if { [info exists Reg_WindowMenuButton($menuButtonName)] && \
               [lsearch $Reg_WindowMenuButton($menuButtonName) $windowName ] != -1 }  {
            set menubutton_command [Create_MenuButton $menuButtonName $object "undocked"]
            set menuButtonObj [eval $menubutton_command]

         }         
      }
      set windowMenuBar($windowName) $object
   }
   return $windowMenuBar($windowName)   
} 

   
itcl::body MenuBarManager::CallBackHandler { menuName menuButtonName } {
   set activeWindowTag "yes"

   # first we get the "windowName" which is currently selected
   set activeWindow [$frameworkHandle getCurrentChildWindow]
   
   set windowIndex [lsearch $Reg_WindowMenu($menuButtonName,$menuName) $activeWindow]
    
   if { $windowIndex == -1 }  {
      set activeWindowTag "no"
      ## setting the window-index for "frameworkWindow"
      set windowIndex  [lsearch $Reg_WindowMenu($menuButtonName,$menuName) "frameworkWindow" ]
   }

   # here we call the respective method
   set callBackData [lindex $Reg_WindMenuCallback($menuButtonName,$menuName) $windowIndex]
   set functionName [lindex $callBackData 0]
   
   if { $activeWindowTag == "yes" } {
      set functionArgs "::DftDebugFramework::$activeWindow $functionName"
   } else {
      set functionArgs "$frameworkHandle $functionName"
   }

   if { [lreplace $callBackData 0 0 ] != "" } {
      set counter 1
      while { $counter < [llength $callBackData] } { 
        lappend functionArgs [lindex $callBackData $counter]
        incr counter
      }
   }
      
   eval [join $functionArgs]
   
} 
           
 
itcl::body MenuBarManager::UpdateState { state windowName menuButtonName { menuName "" } } {

   # Author     : Subrat Panda
   # Description: Fix for DR 765659
   # Note       : Ideally, we should update the menubar of windowName only when windowName is currently active
   if {$windowName != "" && [$frameworkHandle getCurrentChildWindow] != $windowName} {return}
   
   # here we first find all the doc bars associated
   # with "windowName" and send an "update_DocBar" it
   if { $state == "active" } {
      set state "normal"
   }
   
   set paneManager [$frameworkHandle getPaneManager]    
   
   ## setting the window state here
   if { $windowName != "frameworkWindow" } {
      set windowState [ $paneManager state "$paneManager.$windowName" ]
   } else {
      set windowState "normal"
   }

   if { $windowState != "undocked" } {
      set object $globalMenuBarObject
      set menuBarObjState "global"
   } else {
      if { [info exists windowMenuBar($windowName)] } {
         set object $windowMenuBar($windowName)
      } else {
         return;
      }
      set menuBarObjState "local"
   } 

   ## if menuName is null then there is a call to enable/disable menubutton
   if { $windowState != "hidden" } {
      if { $menuName != "" } { 
         set fullMenuName [split $menuName ,] 
         set parentMenuName [lindex $fullMenuName 0]
         set menuObj "$object.menubar.$menuButtonName.menu"
         
         if { [llength $fullMenuName] > 1 } {
            set menuIndex 0
            set totalLength [expr [llength $fullMenuName] + -1]
            while { $menuIndex < $totalLength } {
               set interMenuName [lindex $fullMenuName $menuIndex]
               set menuObj "$menuObj\.$interMenuName$menuBarObjState"
               incr menuIndex
            } 
         }
         
         if { ![winfo exists $menuObj] } {
            return;
         }
         
         set end [$menuObj index last]
         if {$end == "none"} {
            set end 0
         }
         if { [lsearch $Reg_WindowMenuButton($menuButtonName) $windowName] != -1  && \
              [lsearch $Reg_WindowMenu($menuButtonName,$parentMenuName) $windowName] != -1 } {
            set index 0
            while {$index <= $end} {
               if { [$menuObj type $index] != "separator" } {
                  set tempMenuName [$menuObj entrycget $index -label]
                  if { [lindex $menuProps($menuButtonName,$menuName) 0] == $tempMenuName } {
                     $menuObj entryconfigure $index -state $state
                     break
                  }
               }
               incr index
           }
        }
      } else {
         if { [info exists Reg_WindowMenuButton($menuButtonName)] && \
              [lsearch $Reg_WindowMenuButton($menuButtonName) $windowName] != -1 } {
            $object menuconfigure $menuButtonName -state $state
         }
      }
   }
   
   return;
 }


itcl::body MenuBarManager::maximizeWindowMenuBars { windowName } {
   
   ## here we highlight all the menu-buttons that is associated
   ## with the "windowName"
   
   foreach menuButtons $globalMenuButtons {
      if { [info exists Reg_WindowMenuButton($menuButtons)] && \
           [lsearch $Reg_WindowMenuButton($menuButtons) $windowName] == -1 && \
           [lsearch $Reg_WindowMenuButton($menuButtons) "frameworkWindow" ] == -1 && \
           $menuButtonStatus($menuButtons) != "hide" } {
         
         ## here we are deleting the menuButton
         $globalMenuBarObject delete $menuButtons
         
         #removing menu-button from created menu-buttons list
         set index [lsearch $createdMenuButtons $menuButtons]
         if { $index != -1 } {
            set createdMenuButtons [lreplace $createdMenuButtons $index $index]
            set createdMenuButtons [linsert $createdMenuButtons $index " "]
         }

         ## setting the status here
         set menuButtonStatus($menuButtons) "hide"
         
      } elseif { [info exists Reg_WindowMenuButton($menuButtons)] && \
                 [lsearch $Reg_WindowMenuButton($menuButtons) $windowName] != -1 } {
         ## here we are enabling the menu-button
         
         ## if the menubutton was hidden, then we will 
         ## have to create a new-one, since we have deleted it
         if { $menuButtonStatus($menuButtons) == "hide"  && \
              [lsearch $createdMenuButtons $menuButtonName]  == -1 } {
            set menubutton_command [Create_MenuButton $menuButtons $globalMenuBarObject "undocked"]
            set menuButtonObj [eval $menubutton_command]
         }

         $globalMenuBarObject menuconfigure $menuButtons -state normal
         set menuButtonStatus($menuButtons) "enabled"
         
         Update_MenuButton $menuButtons $globalMenuBarObject $windowName "global"
      }
   }
}
 

## this function stores the "value" in the widget's variable
itcl::body MenuBarManager::setWidgetVal { windowName menuButtonName menuName value } {   
   set underscore "_"
   
   if { [lindex $menuProps($menuButtonName,$menuName) 2] == "radiobutton" || [lindex $menuProps($menuButtonName,$menuName) 2] == "checkbutton" } {
      set menuTail [lindex $menuProps($menuButtonName,$menuName) 3]
      global var$underscore$menuTail
      global var$underscore$menuTail$underscore$windowName
   } else {
      global var$underscore$menuButtonName$underscore$menuName
      global var$underscore$menuButtonName$underscore$menuName$underscore$windowName
   }    

   set paneManager [$frameworkHandle getPaneManager]
 
   ## setting the window state here
   if { $windowName != "frameworkWindow" } {
      set windowManaged [$paneManager panes]
      if { $windowManaged != "" } {
         set windowState [ $paneManager state "$paneManager.$windowName" ]
      } else {
         set windowState "normal" 
      }
   }  else {
      set windowState "normal"
   }

   if { $windowState != "undocked" } {
      if { [lindex $menuProps($menuButtonName,$menuName) 2] == "radiobutton" || [lindex $menuProps($menuButtonName,$menuName) 2] == "checkbutton" } {
         set menuTail [lindex $menuProps($menuButtonName,$menuName) 3]
         set var$underscore$menuTail $value
      } else {
         set var$underscore$menuButtonName$underscore$menuName $value
      }
   } else {
      if { [lindex $menuProps($menuButtonName,$menuName) 2] == "radiobutton" || [lindex $menuProps($menuButtonName,$menuName) 2] == "checkbutton" } {
         set menuTail [lindex $menuProps($menuButtonName,$menuName) 3]
         set var$underscore$menuTail$underscore$windowName $value
      } else {
         set var$underscore$menuButtonName$underscore$menuName$underscore$windowName $value
      }
   }
}  


## this function returns the "value" in the widget's variable
itcl::body MenuBarManager::getWidgetVal { windowName menuButtonName menuName } {   
   set underscore "_"
   
   if { [lindex $menuProps($menuButtonName,$menuName) 2] == "radiobutton" || [lindex $menuProps($menuButtonName,$menuName) 2] == "checkbutton" } {
      set menuTail [lindex $menuProps($menuButtonName,$menuName) 3]
      global var$underscore$menuTail
      global var$underscore$menuTail$underscore$windowName
   } else {
      global var$underscore$menuButtonName$underscore$menuName
      global var$underscore$menuButtonName$underscore$menuName$underscore$windowName
   } 


   set paneManager [$frameworkHandle getPaneManager]
   set comboTag 0
 
   ## setting the window state here
   if { $windowName != "frameworkWindow" } {
      set windowManaged [$paneManager panes]
      if { $windowManaged != "" } {
         set windowState [ $paneManager state "$paneManager.$windowName" ]
      } else {
         set windowState "normal" 
      }
   }  else {
      set windowState "normal"
   }
   
   if { $windowState != "undocked" } {
      if { [lindex $menuProps($menuButtonName,$menuName) 2] == "radiobutton" || [lindex $menuProps($menuButtonName,$menuName) 2] == "checkbutton" } {
         set menuTail [lindex $menuProps($menuButtonName,$menuName) 3]
         set tempStore "var$underscore$menuTail"
      } else {
         set tempstore "var$underscore$menuButtonName$underscore$menuName"
      }
   } else {
      if { [lindex $menuProps($menuButtonName,$menuName) 2] == "radiobutton" || [lindex $menuProps($menuButtonName,$menuName) 2] == "checkbutton" } {
         set menuTail [lindex $menuProps($menuButtonName,$menuName) 3]
         set tempStore "var$underscore$menuTail$underscore$windowName"
      } else {
         set tempstore "var$underscore$menuButtonName$underscore$menuName$underscore$windowName"
      } 
   }
   
   set returnVal [set $tempStore]
   return $returnVal
   
}

itcl::body MenuBarManager::MenuItemValidForData { args } {
  
   set validTag 0   

   switch $args {
    Test_Setup    {
       set validTag [c_DoesProcExist TestSetup]
    }
    
    Test_End   {
       set validTag [c_DoesProcExist TestEnd]
    }
    
    Load_Unload  {
       set validTag [c_DoesProcExist LoadUnload]
    }

    Shift  {
       set validTag [c_DoesProcExist Shift]
    }

    Skew_Load  {
       set validTag [c_DoesProcExist SkewLoad]
    }

    Shadow_Control  {
       set validTag [c_DoesProcExist ShadowControl]
    }

    Master_Observe  {
       set validTag [c_DoesProcExist MasterObserve]
    }

    Shadow_Observe  {
       set validTag [c_DoesProcExist ShadowObserve]
    }

    State_Stability {
       set validTag [c_DoesProcExist StateStability]
    }

    Error_Pattern {
       set validTag [c_IsDRCError]
    }

    Assert {
       set validTag [c_IsDRCAssert assert]
    }
    
    Assert1 {
       set validTag [c_IsDRCAssert assert1]
    }
    
    Assert2 {
       set validTag [c_IsDRCAssert assert2]
    }
    
    Assert3 {
       set validTag [c_IsDRCAssert assert3]
    }
    
    K19 { 
       set validTag [c_IsDRCViolation K19]
    }
    
    K22 { 
       set validTag [c_IsDRCViolation K22]
    }

    Pattern_Index   {
       set validTag [c_DoesPatternExist NormalPattern]
    }
   
    Parallel_Pattern {
       set validTag [c_DoesPatternExist ParallelPattern]
    }
    
    Record {return "no"}

    Fault {
       set validTag [c_doesFaultCatExist all] 
    }
    
    SeqDepth {
       set validTag [c_IsSeqDepthValid]
    }

    VCD   {
       set validTag [c_DoesVCDDataExist]
    }
   }
   
   ## returning "value" depending on the 
   ## value of "validTag" 
   if { $validTag == 1 } {
      return "yes"
   } else {
      return "no"
   }

}

itcl::body MenuBarManager::MenuItemValidForBrowser { args } {
   set dataType [lindex $args 0]
   set value [lindex $args 1]
   set result 0

   if { $dataType == "fault" } {
     set result [c_doesFaultCatExist $value]
   } elseif { $dataType == "test" } {
     set faultExists [c_doesFaultCatExist $value]
     set expandableFaults [list AU UC UO DS DI]
	 if {[lsearch $expandableFaults $value] != -1 && $faultExists == 1} {
        set result 1
     } else {
        set result 0
	 }
   } elseif { $dataType == "drc" } {
     set result [c_IsDRCViolation $value]
   } elseif { $dataType == "atpg_eff" } {
     set result [c_doesATPGeffExist]
   } elseif { $dataType == "fault_cov" } {
     set result [c_doesFaultCovExist]
   } elseif { $dataType == "test_cov" || $dataType == "rel_test_cov" } {
     set result [c_doesTestCovExist]
   } elseif { $dataType == "test_cov_loss" } {
     set result [c_doesTestCovLossExist]
   } elseif { $dataType == "coverage" } {
     set result [c_doesCoverageExist]
   }

   if {$result == "1" } {
      return "yes"
   }
   return "no"
}

itcl::body MenuBarManager::MenuItemValidForMarking { args } {
   set dataType [lindex $args 0]
   set value [lindex $args 1]
   set result 0

   if { $dataType != "marking" } {return "no"}

   global VisualizerPrefs
   if { $value == "color1" && $VisualizerPrefs($currentWindow,colorMarkedCount) >= 1} {
      lset menuProps(display,marking,color1) 0 "[string toupper $VisualizerPrefs($currentWindow,colorMarked,1) 0]"
      return "yes"
   } elseif { $value == "color2" && $VisualizerPrefs($currentWindow,colorMarkedCount) >= 2} {
      lset menuProps(display,marking,color2) 0 "[string toupper $VisualizerPrefs($currentWindow,colorMarked,2) 0]"
      return "yes"
   } elseif { $value == "color3" && $VisualizerPrefs($currentWindow,colorMarkedCount) >= 3} {
      lset menuProps(display,marking,color3) 0 "[string toupper $VisualizerPrefs($currentWindow,colorMarked,3) 0]"
      return "yes"
   } elseif { $value == "color4" && $VisualizerPrefs($currentWindow,colorMarkedCount) >= 4} {
      lset menuProps(display,marking,color4) 0 "[string toupper $VisualizerPrefs($currentWindow,colorMarked,4) 0]"
      return "yes"
   } elseif { $value == "color5" && $VisualizerPrefs($currentWindow,colorMarkedCount) >= 5} {
      lset menuProps(display,marking,color5) 0 "[string toupper $VisualizerPrefs($currentWindow,colorMarked,5) 0]"
      return "yes"
   } elseif { $value == "unmark"} {
      return "yes"
   }
            
   return "no"
}


itcl::body MenuBarManager::setKeyIndex {menuNameList} {
   set keyBindingList [list]
   
   foreach ele $menuNameList {
      set idx 0
      set len [string length $ele]
      for { set idx 0 } { $idx < $len } { incr idx } {
         set charVal [string index $ele $idx]
         set toLowerCharVal [string tolower $charVal] 
         set toUpperCharVal [string toupper $charVal] 
         
         if { [lsearch -exact $keyBindingList $toLowerCharVal] == -1  && \
            [lsearch -exact $keyBindingList $toUpperCharVal] == -1 } {
            break
         }
      }
      lappend keyBindingList $charVal
      set keyIndexArray($ele) $idx
   }
}

itcl::body MenuBarManager::getKeyIndex {menuName} {
   if  { [info exists keyIndexArray] && [info exists keyIndexArray($menuName)] } {
      return $keyIndexArray($menuName) 
   }
   return ""
}


